@@ -29,6 +29,7 @@ extern "C" {
 
 #include "ivtv-ext-api.h"
 }
+#include "blank.h"
 
 
 #define UNSYNCED 0
@@ -146,23 +147,26 @@ cPvr350Device::cPvr350Device(void)
     }
 
     mute = 0;
+    spuDecoder = NULL;
     
     /* Create a new ringbuffer for queuing data up into, and create
        the writing thread
     */
     state = UNSYNCED;
     m_RingBuffer = new cRingBufferLinear(3000 * 1024, 0, true);
-    new cPvr350Writer(m_RingBuffer,fd_out);
+    writer = new cPvr350Writer(m_RingBuffer,fd_out);
 }
 
 cPvr350Device::~cPvr350Device()
 {
-    Cancel();
+   delete writer;
 }
 
 void cPvr350Device::MakePrimaryDevice(bool On)
 {
-    printf("Now primary\n");
+#if VDRVERSNUM >= 10307
+    new cPvr350OsdProvider(fbfd, osdbuffer);
+#endif
 }
 
 
@@ -186,7 +190,6 @@ bool cPvr350Device::SetPlayMode(ePlayMode PlayMode)
 { 
     int          ret;
     m_PlayMode = PlayMode;
-    printf("Set playmode called %d\n",PlayMode);
     switch ( PlayMode ) {
     case pmNone:
         struct ivtv_ioctl_fwapi   fwcall;
@@ -199,8 +202,14 @@ bool cPvr350Device::SetPlayMode(ePlayMode PlayMode)
         if ((ret = ioctl(fd_out, IVTV_IOC_FWAPI, &fwcall)) < 0) {
             printf("STOP_DECODE %d\n",ret);
         }
+        if ( ( ret = ioctl(fd_out, IVTV_IOC_DEC_FLUSH, 0) ) < 0 ) {
+            printf("4: %d\n",ret);
+        }
+        Clear();
         break;
+    case pmAudioOnlyBlack:
     default:    
+	PlayVideo(blank,sizeof(blank));
         ivtv_cfg_start_decode startd;
         Clear();
 
@@ -232,6 +241,7 @@ void cPvr350Device::TrickSpeed(int Speed)
     if ((ret = ioctl(fd_out, IVTV_IOC_FWAPI, &fwcall)) < 0) {
         printf("STOP_DECODE %d\n",ret);
     }
+    Clear();
     SetPlayMode(m_PlayMode);
 }
 
@@ -273,7 +283,6 @@ void cPvr350Device::Freeze(void)
 
 void cPvr350Device::Mute(void) 
 {
-    printf("Mute called, but we can't mute\n");
 }
 
 /* This bit of code is taken from softdevice - simply split up the audio/video
@@ -298,8 +307,10 @@ int cPvr350Device::PlayVideo(const uchar *Data, int Length)
                 SetVidInfo(inbuf_ptr,size);
             } else if ( (syncword == 0x000001C0)  && (syncword <= 0x000001CF) ) {
                 state=PAYLOAD;
-                streamtype= syncword- 0x00000100;
-            } else if ( ( syncword == 0x000001b3) ) {
+                streamtype= 0xC0; // syncword- 0x00000100;
+            } else if ( syncword == 0x000001bd)  {  /* ac3 */
+                state=PAYLOAD;
+                streamtype= 0xC0; // syncword- 0x00000100;
                
             }
             break;
@@ -337,12 +348,18 @@ int cPvr350Device::PlayVideo(const uchar *Data, int Length)
 
 void cPvr350Device::SetVolumeDevice(int Volume)
 {
-    printf("Volume called %d, but we can't mute\n",Volume);
 }
 
 void cPvr350Device::StillPicture(const uchar *Data, int Length) 
 {
-    printf("StillPicture not implemented yet...\n");
+#if 0
+#define MIN_IFRAME 400000
+    for (int i = MIN_IFRAME / Length + 1; i > 0; i--) {
+        safe_write(fd_out, Data, Length);
+        usleep(1); // allows the buffer to be displayed in case the progress display is active
+    }
+#endif
+
 }
 
 bool cPvr350Device::Poll(cPoller &Poller, int TimeoutMs) 
@@ -351,19 +368,22 @@ bool cPvr350Device::Poll(cPoller &Poller, int TimeoutMs)
     return Poller.Poll(TimeoutMs);
 }
 
+
+cSpuDecoder *cPvr350Device::GetSpuDecoder(void)
+{
+    if (!spuDecoder && IsPrimaryDevice())
+        spuDecoder = new cDvbSpuDecoder();
+    return spuDecoder;
+}
+
+#if VDRVERSNUM < 10307
 cOsdBase *cPvr350Device::NewOsd(int x, int y)
 {
     return new cPvr350Osd(fbfd,osdbuffer,x,y);
 }
+#endif
 
 
-cSpuDecoder *cPvr350Device::GetSpuDecoder(void)
-{
-          if (!spuDecoder && IsPrimaryDevice())
-                       spuDecoder = new cDvbSpuDecoder();
-            return spuDecoder;
-}
-
 void cPvr350Device::SetVidInfo(const uchar *mbuf, int count)
 {
     const uchar *headr;
@@ -402,10 +422,8 @@ void cPvr350Device::SetVidInfo(const uchar *mbuf, int count)
     
     switch ( sw ) {
     case 1:
-        fprintf(stderr,"Videostream: ASPECT: 1:1");        
         break;
     case 2:
-        fprintf(stderr,"Videostream: ASPECT: 4:3");
         if ( horizontal_size < vertical_size) {
             fwcall.data[0] = 720;
             fwcall.data[1] = 576;
@@ -428,7 +446,6 @@ void cPvr350Device::SetVidInfo(const uchar *mbuf, int count)
         }
         break;
     case 3:
-        fprintf(stderr,"Videostream: ASPECT: 16:9");
         if ( horizontal_size < vertical_size) {
             fwcall.data[0] = 720;
             fwcall.data[1] = (720 / 16) * 9;
@@ -451,14 +468,16 @@ void cPvr350Device::SetVidInfo(const uchar *mbuf, int count)
         }
         break;
     case 4:
-        fprintf(stderr,"Videostream: ASPECT: 2.21:1");
         break;                    
     case 5 ... 15:
-        fprintf(stderr,"Videostream: ASPECT: reserved");
         break;
     default:
-        printf("Unknown aspect\n");
         break;
     }
-    printf("Size is %d x %d\n",horizontal_size,vertical_size);
 }
+
+int64_t cPvr350Device::GetSTC(void)
+{
+    return -1;
+}
+
