@@ -11,6 +11,7 @@ cPvr350Writer::cPvr350Writer(cRingBufferLinear *ring, int fd)
     output = fd;
 
     m_RingBuffer = ring;
+    running = true;
 
     Start();
 }
@@ -19,6 +20,7 @@ cPvr350Writer::cPvr350Writer(cRingBufferLinear *ring, int fd)
 
 cPvr350Writer::~cPvr350Writer()
 {
+    running = false;
     Cancel();
 }
 
@@ -29,10 +31,10 @@ void cPvr350Writer::Action()
     unsigned char *buf;
     int length;
     printf("Writer started pid %d\n",getpid());
-        while(1) {
+        while(running) {
         if ( (buf = m_RingBuffer->Get(length) ) != NULL ) {
             /* Writing a length of 4000 seems to keep ivtv happy */
-            if ( length >= 4000 ) {
+            if (  length >= 4000 ) {
                 length = 4000;
                 safe_write(output,buf,length);                           
                 m_RingBuffer->Del(length);  
